# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/tools/api/generator/create_python_api.py script.
"""Home of estimator related functions.

"""

from __future__ import print_function

from tensorflow.python.estimator.estimator_lib import model_to_estimator

del print_function
