# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/tools/api/generator/create_python_api.py script.
"""Public API for tf.initializers namespace.
"""

from __future__ import print_function

from tensorflow.python import Constant as constant
from tensorflow.python import Identity as identity
from tensorflow.python import Ones as ones
from tensorflow.python import Orthogonal as orthogonal
from tensorflow.python import RandomNormal as random_normal
from tensorflow.python import RandomUniform as random_uniform
from tensorflow.python import TruncatedNormal as truncated_normal
from tensorflow.python import UniformUnitScaling as uniform_unit_scaling
from tensorflow.python import VarianceScaling as variance_scaling
from tensorflow.python import Zeros as zeros
from tensorflow.python import global_variables_initializer as global_variables
from tensorflow.python import local_variables_initializer as local_variables
from tensorflow.python import variables_initializer as variables

del print_function
