# LD_LIBRARY_PATH needs to be adjusted to find the libcuda.so file
# This may change depending on the system configuration.  
# This path is for the nvidia/cuda:7.5-cudnn5-devel-centos6 container
LD_LIBRARY_PATH="/usr/local/nvidia/lib64:$LD_LIBRARY_PATH" python gpu_test.py


set -ex



freeze_graph --help
saved_model_cli --help
tflite_convert --help
toco_from_protos --help
toco --help
conda inspect linkages -p $PREFIX tensorflow-base
exit 0
