# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/tools/api/generator/create_python_api.py script.
"""ResNet50 model for Keras.

# Reference:

- [Deep Residual Learning for Image
Recognition](https://arxiv.org/abs/1512.03385)

Adapted from code contributed by BigMoyan.

"""

from __future__ import print_function

from tensorflow.python.keras.applications import ResNet50
from tensorflow.python.keras.applications.densenet import decode_predictions
from tensorflow.python.keras.applications.imagenet_utils import preprocess_input

del print_function
